@@ -1,17 +1,17 @@
 import daemon
 import signal
+import socket
 import sys
 import pid
 from datetime import datetime
 import builtins
-from contextlib import redirect_stdout,redirect_stderr
 from systemd.daemon import notify
 
 
 class Service:
     def __init__(self, name, **kwargs):
         self.name = name
-        # argsparse?
+        # may be argsparse later?
         self.systemd = True if '-systemd' in sys.argv else False
         self.not_daemonize = True if '-not-daemonize' in sys.argv else False
 
@@ -29,10 +29,10 @@ class Service:
                                              stdout=self.f_log,
                                              stderr=self.f_err)
         self.dcontext.signal_map = {
-            signal.SIGTERM: self.exit_proc,
+            signal.SIGTERM: self.exit_proc,  #
+            signal.SIGINT: self.exit_proc,  # Ctrl + c reaction
         }
 
-        redef_print = kwargs.get('redef_print', False)
         self.print = builtins.print
         builtins.print = self.log_str
 
@@ -40,13 +40,15 @@ class Service:
             print('starting service: ' + self.name)
             self.pre_run()
             self.main()
-            notify('READY=1')
+            if self.systemd:
+                notify('READY=1')
             print('data loaded, going to main loop')
             self.run_main_loop()
 
     def exit_proc(self, signum, frame):
         self.log_str('signal recieved: %d, %s' % (signum, frame))
-        notify('STOPPING=1')
+        if self.systemd:
+            notify('STOPPING=1')
         self.clean_proc()
         self.quit_main_loop()
 
@@ -113,9 +115,28 @@ class CXService(Service):
     def run_main_loop(self):
         global cda
         import pycx4.pycda as cda
+
+        # Create a socket pair for waking up from cda's select()
+        self.wsock, self.rsock = socket.socketpair(type=socket.SOCK_DGRAM)
+        self.wsock.setblocking(False) # required by signal library
+        self.old_fd = signal.set_wakeup_fd(self.wsock.fileno())
+        # Create Fd event fpr CX scheduler main loop
+        self.wakeup_ev = cda.FdEvent(self.rsock)
+        # call to self.wakeup_proc will return execution to python interpriter
+        # on tests after that python's signal processing called first, then
+        # self.wakeup_proc
+        self.wakeup_ev.ready.connect(self.wakeup_proc)
+        # may be change to dummy handler lake that:
+        #self.wakeup_ev.ready.connect(lambda : None)
+        # since really not required to processsignals in wakeup proc
+
         cda.main_loop()
 
     def quit_main_loop(self):
         global cda
         import pycx4.pycda as cda
         cda.break_()
+
+    def wakeup_proc(self, ev):
+        data = self.rsock.recv(1)
+        print('wake up with:', data)
